/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core;

import org.apache.maven.project.MavenProject;

import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.BuildConfiguratorExtensionConfiguration;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.BuildConfiguratorPluginConfiguration;

/**
 * Contains parameters for configuring a project.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class ConfigureRequest
{
    /**
     * Builder for building {@link ConfigureRequest} objects.
     *
     * @author htfv (Aliaksei Lahachou)
     */
    public static class Builder
    {
        /**
         * Flag indicating whether configuration request is coming from m2e
         * connector.
         */
        private boolean connectorRequest;

        /**
         * {@link BuildConfiguratorExtensionConfiguration} with configuration
         * parameters.
         */
        private BuildConfiguratorExtensionConfiguration extensionConfiguration;

        /**
         * {@link BuildConfiguratorPluginConfiguration} with configuration
         * parameters.
         */
        private BuildConfiguratorPluginConfiguration pluginConfiguration;

        /**
         * {@link MavenProject} to configure.
         */
        private MavenProject project;

        /**
         * Build a new {@link ConfigureRequest} object.
         *
         * @return a new {@link ConfigureRequest} object.
         */
        public ConfigureRequest build()
        {
            return new ConfigureRequest(this);
        }

        /**
         * Sets the flag indicating whether configuration request is coming from
         * m2e connector to {@code true}.
         *
         * @return {@code this}.
         */
        public Builder connectorRequest()
        {
            return connectorRequest(true);
        }

        /**
         * Sets the flag indicating whether configuration request is coming from
         * m2e connector.
         *
         * @param pConnectorRequest
         *            {@code true} if request comes from m2e connector;
         *            {@code false} otherwise.
         *
         * @return {@code this}.
         */
        public Builder connectorRequest(final boolean pConnectorRequest)
        {
            this.connectorRequest = pConnectorRequest;
            return this;
        }

        /**
         * Sets {@link BuildConfiguratorExtensionConfiguration} and
         * {@link BuildConfiguratorPluginConfiguration} with configuration
         * parameters.
         *
         * @param pExtensionConfiguration
         *            {@link BuildConfiguratorExtensionConfiguration} with
         *            configuration parameters.
         *
         * @return {@code this}.
         */
        public Builder extensionConfiguration(
                final BuildConfiguratorExtensionConfiguration pExtensionConfiguration)
        {
            this.extensionConfiguration = pExtensionConfiguration;
            this.pluginConfiguration    = pExtensionConfiguration;

            return this;
        }

        /**
         * Sets {@link BuildConfiguratorPluginConfiguration} with configuration
         * parameters, resets {@link BuildConfiguratorExtensionConfiguration} to
         * {@code null}.
         *
         * @param pPluginConfiguration
         *            {@link BCPlugniModel} with configuration parameters.
         *
         * @return {@code this}.
         */
        public Builder pluginConfiguration(
                final BuildConfiguratorPluginConfiguration pPluginConfiguration)
        {
            this.extensionConfiguration = null;
            this.pluginConfiguration    = pPluginConfiguration;

            return this;
        }

        /**
         * Sets {@link MavenProject} to configure.
         *
         * @param pProject
         *            {@link MavenProject} to configure.
         *
         * @return {@code this}.
         */
        public Builder project(final MavenProject pProject)
        {
            this.project = pProject;
            return this;
        }
    }

    /**
     * Flag indicating whether configuration request is coming from m2e
     * connector.
     */
    private final boolean connectorRequest;

    /**
     * {@link BuildConfiguratorExtensionConfiguration} with configuration
     * parameters.
     */
    private final BuildConfiguratorExtensionConfiguration extensionConfiguration;

    /**
     * {@link BuildConfiguratorPluginConfiguration} with configuration
     * parameters.
     */
    private final BuildConfiguratorPluginConfiguration pluginConfiguration;

    /**
     * {@link MavenProject} to configure.
     */
    private final MavenProject project;

    /**
     * Constructs a new {@link ConfigureRequest} object.
     *
     * @param builder
     *            {@link Builder} containing parameters for construction.
     */
    protected ConfigureRequest(final Builder builder)
    {
        this.connectorRequest       = builder.connectorRequest;
        this.extensionConfiguration = builder.extensionConfiguration;
        this.pluginConfiguration    = builder.pluginConfiguration;
        this.project                = builder.project;
    }

    /**
     * Gets the {@link BuildConfiguratorExtensionConfiguration} with
     * configuration parameters.
     *
     * @return {@link BuildConfiguratorExtensionConfiguration} with
     *         configuration parameters.
     */
    public BuildConfiguratorExtensionConfiguration getExtensionConfiguration()
    {
        return extensionConfiguration;
    }

    /**
     * Gets the {@link BuildConfiguratorPluginConfiguration} with configuration
     * parameters.
     *
     * @return {@link BuildConfiguratorPluginConfiguration} with configuration
     *         parameters.
     */
    public BuildConfiguratorPluginConfiguration getPluginConfiguration()
    {
        return pluginConfiguration;
    }

    /**
     * Gets the {@link MavenProject} to configure.
     *
     * @return {@link MavenProject} to configure.
     */
    public MavenProject getProject()
    {
        return project;
    }

    /**
     * Gets the flag indicating whether configuration request is coming from m2e
     * connector.
     *
     * @return {@code true} if request comes from m2e connector; {@code false}
     *         otherwise.
     */
    public boolean isConnectorRequest()
    {
        return connectorRequest;
    }
}
