/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core;

import org.apache.maven.execution.ExecutionEvent;
import org.apache.maven.execution.ExecutionListener;

/**
 * {@link ExecutionListener} which delegates all calls to another object.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class DelegatingExecutionListener implements ExecutionListener
{
    /**
     * {@link ExecutionListener} to which all calls are delegated.
     */
    private final ExecutionListener delegate;

    /**
     * Constructs a new {@link DelegatingExecutionListener} object.
     *
     * @param delegate
     *            {@link ExecutionListener} to which all calls are delegated.
     */
    public DelegatingExecutionListener(final ExecutionListener delegate)
    {
        this.delegate = delegate;
    }

    @Override
    public void forkedProjectFailed(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.forkedProjectFailed(event);
        }
    }

    @Override
    public void forkedProjectStarted(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.forkedProjectStarted(event);
        }
    }

    @Override
    public void forkedProjectSucceeded(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.forkedProjectSucceeded(event);
        }
    }

    @Override
    public void forkFailed(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.forkFailed(event);
        }
    }

    @Override
    public void forkStarted(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.forkStarted(event);
        }
    }

    @Override
    public void forkSucceeded(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.forkSucceeded(event);
        }
    }

    /**
     * Gets {@link ExecutionListener} to which all calls are delegated.
     *
     * @return {@link ExecutionListener} to which all calls are delegated.
     */
    public ExecutionListener getDelegate()
    {
        return delegate;
    }

    @Override
    public void mojoFailed(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.mojoFailed(event);
        }
    }

    @Override
    public void mojoSkipped(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.mojoSkipped(event);
        }
    }

    @Override
    public void mojoStarted(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.mojoStarted(event);
        }
    }

    @Override
    public void mojoSucceeded(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.mojoSucceeded(event);
        }
    }

    @Override
    public void projectDiscoveryStarted(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.projectDiscoveryStarted(event);
        }
    }

    @Override
    public void projectFailed(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.projectFailed(event);
        }
    }

    @Override
    public void projectSkipped(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.projectSkipped(event);
        }
    }

    @Override
    public void projectStarted(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.projectStarted(event);
        }
    }

    @Override
    public void projectSucceeded(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.projectSucceeded(event);
        }
    }

    @Override
    public void sessionEnded(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.sessionEnded(event);
        }
    }

    @Override
    public void sessionStarted(final ExecutionEvent event)
    {
        if (delegate != null)
        {
            delegate.sessionStarted(event);
        }
    }
}
