/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.maven.model.Build;
import org.apache.maven.model.Dependency;
import org.apache.maven.model.Exclusion;
import org.apache.maven.model.Extension;
import org.apache.maven.model.Model;
import org.apache.maven.model.Plugin;
import org.apache.maven.model.PluginExecution;
import org.apache.maven.model.PluginManagement;
import org.apache.maven.model.Resource;
import org.codehaus.plexus.interpolation.InterpolationException;
import org.codehaus.plexus.interpolation.Interpolator;
import org.codehaus.plexus.interpolation.StringSearchInterpolator;
import org.codehaus.plexus.util.xml.Xpp3Dom;

/**
 * Traverses the model and reinterpolates elements.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class ModelInterpolator
{
    /**
     * Interpolator used to interpolate single strings.
     */
    private final Interpolator interpolator;

    /**
     * Constructs a new {@link ModelInterpolator} object.
     *
     * @param properties
     *            map supplying properties for interpolation.
     */
    public ModelInterpolator(final Map<String, String> properties)
    {
        this.interpolator = new StringSearchInterpolator();

        interpolator.addValueSource(new PropertiesValueSource(properties));
        interpolator.setCacheAnswers(true);
    }

    /**
     * Interpolates elements of the {@link Build} object.
     *
     * @param build
     *            {@link Build} object whose elements to interpolate.
     */
    private void interpolateBuild(final Build build)
    {
        build.setDefaultGoal(
                interpolateString(build.getDefaultGoal()));
        build.setDirectory(
                interpolateString(build.getDirectory()));
        build.setFinalName(
                interpolateString(build.getFinalName()));
        build.setOutputDirectory(
                interpolateString(build.getOutputDirectory()));
        build.setScriptSourceDirectory(
                interpolateString(build.getScriptSourceDirectory()));
        build.setSourceDirectory(
                interpolateString(build.getSourceDirectory()));
        build.setTestOutputDirectory(
                interpolateString(build.getTestOutputDirectory()));
        build.setTestSourceDirectory(
                interpolateString(build.getTestSourceDirectory()));

        interpolateExtensions(build.getExtensions());
        interpolateStrings(build.getFilters());
        interpolatePluginManagement(build.getPluginManagement());
        interpolatePlugins(build.getPlugins());
        interpolateResources(build.getResources());
        interpolateResources(build.getTestResources());
    }

    /**
     * Interpolates elements of the {@link Dependency} objects.
     *
     * @param dependencies
     *            collection of {@link Dependency} objects whose elements to
     *            interpolate.
     */
    private void interpolateDependencies(final Collection<Dependency> dependencies)
    {
        for (Dependency dependency : dependencies)
        {
            interpolateDependency(dependency);
        }
    }

    /**
     * Interpolates elements of the {@link Dependency} object.
     *
     * @param dependency
     *            {@link Dependency} object whose elements to interpolate.
     */
    private void interpolateDependency(final Dependency dependency)
    {
        dependency.setArtifactId(
                interpolateString(dependency.getArtifactId()));
        dependency.setClassifier(
                interpolateString(dependency.getClassifier()));
        dependency.setGroupId(
                interpolateString(dependency.getGroupId()));
        dependency.setOptional(
                interpolateString(dependency.getOptional()));
        dependency.setScope(
                interpolateString(dependency.getScope()));
        dependency.setSystemPath(
                interpolateString(dependency.getSystemPath()));
        dependency.setType(
                interpolateString(dependency.getType()));
        dependency.setVersion(
                interpolateString(dependency.getVersion()));

        interpolateExclusions(dependency.getExclusions());
    }

    /**
     * Interpolates elements of the {@link Xpp3Dom} object.
     *
     * @param dom
     *            {@link Xpp3Dom} object whose elements to interpolate.
     */
    public void interpolateDOM(final Xpp3Dom dom)
    {
        if (dom == null)
        {
            return;
        }

        dom.setValue(interpolateString(dom.getValue()));

        for (Xpp3Dom child : dom.getChildren())
        {
            interpolateDOM(child);
        }
    }

    /**
     * Interpolates elements of the {@link Exclusion} object.
     *
     * @param exclusion
     *            {@link Exclusion} object whose elements to interpolate.
     */
    private void interpolateExclusion(final Exclusion exclusion)
    {
        exclusion.setArtifactId(
                interpolateString(exclusion.getArtifactId()));
        exclusion.setGroupId(
                interpolateString(exclusion.getGroupId()));
    }

    /**
     * Interpolates elements of the {@link Exclusion} objects.
     *
     * @param exclusions
     *            collection of {@link Exclusion} objects whose elements to
     *            interpolate.
     */
    private void interpolateExclusions(final Collection<Exclusion> exclusions)
    {
        for (Exclusion exclusion : exclusions)
        {
            interpolateExclusion(exclusion);
        }
    }

    /**
     * Interpolates elements of the {@link PluginExecution} object.
     *
     * @param execution
     *            {@link PluginExecution} object whose elements to interpolate.
     */
    private void interpolateExecution(final PluginExecution execution)
    {
        execution.setId(
                interpolateString(execution.getId()));
        execution.setInherited(
                interpolateString(execution.getInherited()));
        execution.setPhase(
                interpolateString(execution.getPhase()));

        interpolateDOM((Xpp3Dom) execution.getConfiguration());
        interpolateStrings(execution.getGoals());
    }

    /**
     * Interpolates elements of the {@link PluginExecution} objects.
     *
     * @param executions
     *            collection of {@link PluginExecution} objects whose elements
     *            to interpolate.
     */
    private void interpolateExecutions(final Collection<PluginExecution> executions)
    {
        for (PluginExecution execution : executions)
        {
            interpolateExecution(execution);
        }
    }

    /**
     * Interpolates elements of the {@link Extension} object.
     *
     * @param extension
     *            {@link Extension} object whose elements to interpolate.
     */
    private void interpolateExtension(final Extension extension)
    {
        extension.setArtifactId(
                interpolateString(extension.getArtifactId()));
        extension.setGroupId(
                interpolateString(extension.getGroupId()));
        extension.setVersion(
                interpolateString(extension.getVersion()));
    }

    /**
     * Interpolates elements of the {@link Extension} objects.
     *
     * @param extensions
     *            collection of {@link Extension} objects whose elements to
     *            interpolate.
     */
    private void interpolateExtensions(final Collection<Extension> extensions)
    {
        for (Extension extension : extensions)
        {
            interpolateExtension(extension);
        }
    }

    /**
     * Interpolates elements of the {@link Model} object.
     *
     * @param model
     *            {@link Model} object whose elements to interpolate.
     */
    public void interpolateModel(final Model model)
    {
        interpolateBuild(model.getBuild());
    }

    /**
     * Interpolates elements of the {@link Plugin} object.
     *
     * @param plugin
     *            {@link Plugin} object whose elements to interpolate.
     */
    private void interpolatePlugin(final Plugin plugin)
    {
        plugin.setArtifactId(
                interpolateString(plugin.getArtifactId()));
        plugin.setExtensions(
                interpolateString(plugin.getExtensions()));
        plugin.setGroupId(
                interpolateString(plugin.getGroupId()));
        plugin.setInherited(
                interpolateString(plugin.getInherited()));
        plugin.setVersion(
                interpolateString(plugin.getVersion()));

        interpolateDOM((Xpp3Dom) plugin.getConfiguration());
        interpolateDependencies(plugin.getDependencies());
        interpolateExecutions(plugin.getExecutions());
    }

    /**
     * Interpolates elements of the {@link PluginManagement} object.
     *
     * @param pluginManagement
     *            {@link PluginManagement} object whose elements to interpolate.
     */
    private void interpolatePluginManagement(final PluginManagement pluginManagement)
    {
        interpolatePlugins(pluginManagement.getPlugins());
    }

    /**
     * Interpolates elements of the {@link Plugin} objects.
     *
     * @param plugins
     *            Collection of {@link Plugin} objects whose elements to
     *            interpolate.
     */
    public void interpolatePlugins(final Collection<Plugin> plugins)
    {
        for (Plugin plugin : plugins)
        {
            interpolatePlugin(plugin);
        }
    }

    /**
     * Interpolates elements of the {@link Resource} object.
     *
     * @param resource
     *            {@link Resource} object whose elements to interpolate.
     */
    private void interpolateResource(final Resource resource)
    {
        resource.setDirectory(interpolateString(resource.getDirectory()));
    }

    /**
     * Interpolates elements of the {@link Resource} objects.
     *
     * @param resources
     *            collection of {@link Resource} objects whose elements to
     *            interpolate.
     */
    private void interpolateResources(final Collection<Resource> resources)
    {
        for (Resource resource : resources)
        {
            interpolateResource(resource);
        }
    }

    /**
     * Interpolates a string.
     *
     * @param string
     *            string to interpolate.
     *
     * @return interpolated string.
     */
    private String interpolateString(final String string)
    {
        if (StringUtils.isEmpty(string))
        {
            return string;
        }

        try
        {
            return interpolator.interpolate(string);
        }
        catch (InterpolationException e)
        {
            return string;
        }
    }

    /**
     * Interpolates strings.
     *
     * @param strings
     *            collection of strings to interpolate.
     */
    private void interpolateStrings(final List<String> strings)
    {
        for (int i = 0; i < strings.size(); i++)
        {
            strings.set(i, interpolateString(strings.get(i)));
        }
    }
}
