/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configuration;

import java.util.List;

/**
 * Contains parameters for updating a dependency.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class DependencyConfiguration
{
    /**
     * Dependency is added if it's not yet defined.
     */
    public static final String ACTION_ADD = "add";

    /**
     * Dependency is added if it's not yet defined, or updated if it's already
     * defined.
     */
    public static final String ACTION_ADD_UPDATE = "add-update";

    /**
     * Dependency is skipped.
     */
    public static final String ACTION_SKIP = "skip";

    /**
     * Dependency is updated if it's already defined.
     */
    public static final String ACTION_UPDATE = "update";

    public static final String DEFAULT_TYPE = "jar";

    /**
     * Action to perform with this dependency; may take one of the following
     * values: {@link DependencyConfiguration#ACTION_ADD},
     * {@link DependencyConfiguration#ACTION_ADD_UPDATE}, {@link DependencyConfiguration#ACTION_SKIP},
     * {@link DependencyConfiguration#ACTION_UPDATE}.
     */
    private String action;

    /**
     * {@code artifactId} of the dependency to update.
     */
    private String artifactId;

    /**
     * {@code classifier} of the dependency to update.
     */
    private String classifier;

    /**
     * Dependency exclusions.
     */
    private List<ExclusionConfiguration> exclusions;

    /**
     * {@code groupId} of the dependency to update.
     */
    private String groupId;

    /**
     * {@code optional} flag of the dependency to update.
     */
    private String optional;

    /**
     * {@code scope} of the dependency to update.
     */
    private String scope;

    /**
     * {@code systemPath} of the dependency to update.
     */
    private String systemPath;

    /**
     * {@code type} of the dependency to update.
     */
    private String type;

    /**
     * {@code version} of the dependency to update.
     */
    private String version;

    /**
     * Gets the action to perform with this dependency; may take one of the
     * following values: {@link DependencyConfiguration#ACTION_ADD},
     * {@link DependencyConfiguration#ACTION_ADD_UPDATE}, {@link DependencyConfiguration#ACTION_SKIP},
     * {@link DependencyConfiguration#ACTION_UPDATE}.
     *
     * @return action to perform.
     */
    public String getAction()
    {
        return action;
    }

    /**
     * Gets {@code artifactId} of the dependency to update.
     *
     * @return {@code artifactId} of the dependency to update.
     */
    public String getArtifactId()
    {
        return artifactId;
    }

    /**
     * Gets {@code classifier} of the dependency to update.
     *
     * @return {@code classifier} of the dependency to update.
     */
    public String getClassifier()
    {
        return classifier;
    }

    /**
     * Gets dependency exclusions.
     *
     * @return dependency exclusions.
     */
    public List<ExclusionConfiguration> getExclusions()
    {
        return exclusions;
    }

    /**
     * Gets {@code groupId} of the dependency to update.
     *
     * @return {@code groupId} of the dependency to update.
     */
    public String getGroupId()
    {
        return groupId;
    }

    /**
     * Gets {@code optional} flag of the dependency to update.
     *
     * @return {@code optional} flag of the dependency to update.
     */
    public String getOptional()
    {
        return optional;
    }

    /**
     * Gets {@code scope} of the dependency to update.
     *
     * @return {@code scope} of the dependency to update.
     */
    public String getScope()
    {
        return scope;
    }

    /**
     * Gets {@code systemPath} of the dependency to update.
     *
     * @return {@code systemPath} of the dependency to update.
     */
    public String getSystemPath()
    {
        return systemPath;
    }

    /**
     * Gets {@code type} of the dependency to update.
     *
     * @return {@code type} of the dependency to update.
     */
    public String getType()
    {
        return type;
    }

    /**
     * Gets {@code version} of the dependency to update.
     *
     * @return {@code version} of the dependency to update.
     */
    public String getVersion()
    {
        return version;
    }

    /**
     * Sets the action to perform with this dependency; may take one of the
     * following values: {@link DependencyConfiguration#ACTION_ADD},
     * {@link DependencyConfiguration#ACTION_ADD_UPDATE}, {@link DependencyConfiguration#ACTION_SKIP},
     * {@link DependencyConfiguration#ACTION_UPDATE}.
     *
     * @param action
     *            action to perform.
     */
    public void setAction(final String action)
    {
        this.action = action;
    }

    /**
     * Sets {@code artifactId} of the dependency to update.
     *
     * @param artifactId
     *            {@code artifactId} of the dependency to update.
     */
    public void setArtifactId(final String artifactId)
    {
        this.artifactId = artifactId;
    }

    /**
     * Sets {@code classifier} of the dependency to update.
     *
     * @param classifier
     *            {@code classifier} of the dependency to update.
     */
    public void setClassifier(final String classifier)
    {
        this.classifier = classifier;
    }

    /**
     * Sets dependency exclusions.
     *
     * @param exclusions
     *            dependency exclusions.
     */
    public void setExclusions(final List<ExclusionConfiguration> exclusions)
    {
        this.exclusions = exclusions;
    }

    /**
     * Sets {@code groupId} of the dependency to update.
     *
     * @param groupId
     *            {@code groupId} of the dependency to update.
     */
    public void setGroupId(final String groupId)
    {
        this.groupId = groupId;
    }

    /**
     * Sets {@code optional} flag of the dependency to update.
     *
     * @param optional
     *            {@code optional} flag of the dependency to update.
     */
    public void setOptional(final String optional)
    {
        this.optional = optional;
    }

    /**
     * Sets {@code scope} of the dependency to update.
     *
     * @param scope
     *            {@code scope} of the dependency to update.
     */
    public void setScope(final String scope)
    {
        this.scope = scope;
    }

    /**
     * Sets {@code systemPath} of the dependency to update.
     *
     * @param systemPath
     *            {@code systemPath} of the dependency to update.
     */
    public void setSystemPath(final String systemPath)
    {
        this.systemPath = systemPath;
    }

    /**
     * Sets {@code type} of the dependency to update.
     *
     * @param type
     *            {@code type} of the dependency to update.
     */
    public void setType(final String type)
    {
        this.type = type;
    }

    /**
     * Sets {@code version} of the dependency to update.
     *
     * @param version
     *            {@code version} of the dependency to update.
     */
    public void setVersion(final String version)
    {
        this.version = version;
    }
}
