/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configuration;

import java.util.List;

import org.codehaus.plexus.configuration.PlexusConfiguration;

/**
 * Contains parameters for updating a plugin execution.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class ExecutionConfiguration
{
    /**
     * Execution is added if it's not yet defined.
     */
    public static final String ACTION_ADD = "add";

    /**
     * Execution is added if it's not yet defined, or updated if it's already
     * defined.
     */
    public static final String ACTION_ADD_UPDATE = "add-update";

    /**
     * Execution is skipped.
     */
    public static final String ACTION_SKIP = "skip";

    /**
     * Execution is updated if it's already defined.
     */
    public static final String ACTION_UPDATE = "update";

    /**
     * Action to perform with this execution; may take one of the following
     * values: {@link ExecutionConfiguration#ACTION_ADD},
     * {@link ExecutionConfiguration#ACTION_ADD_UPDATE},
     * {@link ExecutionConfiguration#ACTION_SKIP},
     * {@link ExecutionConfiguration#ACTION_UPDATE}.
     */
    private String action;

    /**
     * Plugin configuration.
     */
    private PlexusConfiguration configuration;

    /**
     * Plugin goals to execute.
     */
    private List<String> goals;

    /**
     * Execution identifier.
     */
    private String id;

    /**
     * Phase to bind the goals in this execution.
     */
    private String phase;

    /**
     * Gets the action to perform with this execution; may take one of the
     * following values: {@link ExecutionConfiguration#ACTION_ADD},
     * {@link ExecutionConfiguration#ACTION_ADD_UPDATE},
     * {@link ExecutionConfiguration#ACTION_SKIP},
     * {@link ExecutionConfiguration#ACTION_UPDATE}.
     *
     * @return action to perform.
     */
    public String getAction()
    {
        return action;
    }

    /**
     * Gets the plugin configuration.
     *
     * @return plugin configuration.
     */
    public PlexusConfiguration getConfiguration()
    {
        return configuration;
    }

    /**
     * Gets the plugin goals to execute.
     *
     * @return plugin goals to execute.
     */
    public List<String> getGoals()
    {
        return goals;
    }

    /**
     * Gets the execution identifier.
     *
     * @return execution identifier.
     */
    public String getId()
    {
        return id;
    }

    /**
     * Gets the phase to bind the goals in this execution.
     *
     * @return phase to bind the goals.
     */
    public String getPhase()
    {
        return phase;
    }

    /**
     * Sets the action to perform with this execution; may take one of the
     * following values: {@link ExecutionConfiguration#ACTION_ADD},
     * {@link ExecutionConfiguration#ACTION_ADD_UPDATE},
     * {@link ExecutionConfiguration#ACTION_SKIP},
     * {@link ExecutionConfiguration#ACTION_UPDATE}.
     *
     * @param action
     *            action to perform.
     */
    public void setAction(final String action)
    {
        this.action = action;
    }

    /**
     * Sets the plugin configuration.
     *
     * @param configuration
     *            plugin configuration.
     */
    public void setConfiguration(final PlexusConfiguration configuration)
    {
        this.configuration = configuration;
    }

    /**
     * Gets the plugin goals to execute.
     *
     * @param goals
     *            plugin goals to execute.
     */
    public void setGoals(final List<String> goals)
    {
        this.goals = goals;
    }

    /**
     * Gets the execution identifier.
     *
     * @param id
     *            execution identifier.
     */
    public void setId(final String id)
    {
        this.id = id;
    }

    /**
     * Gets the phase to bind the goals in this execution.
     *
     * @param phase
     *            phase to bind the goals.
     */
    public void setPhase(final String phase)
    {
        this.phase = phase;
    }
}
