/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configurators;

import java.util.HashMap;
import java.util.Map;

import javax.el.ELContext;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.Configuration;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.project.MavenProject;

import com.github.htfv.maven.plugins.buildconfigurator.core.ConfigureRequest;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.BuildConfiguratorPluginConfiguration;
import com.github.htfv.maven.plugins.buildconfigurator.core.el.BuildConfiguratorELContext;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.ELUtils;

public class ConfigurationContext
{
    private final ELContext elContext;
    private final MavenSession mavenSession;
    private final CompositeConfiguration properties;
    private final ConfigureRequest request;
    private final Map<String, Object> results;

    public ConfigurationContext(final MavenSession mavenSession, final ConfigureRequest request)
    {
        this.mavenSession = mavenSession;
        this.properties   = new CompositeConfiguration();
        this.request      = request;
        this.results      = new HashMap<String, Object>();

        this.elContext = BuildConfiguratorELContext
                .newInstance(request.getProject(), properties);
    }

    public void addProperties(final Configuration properties)
    {
        this.properties.addConfiguration(properties);
    }

    public void addResult(final Class<?> resultType, final Object result)
    {
        results.put(resultType.getName(), result);
    }

    public void addResult(final Object result)
    {
        addResult(result.getClass(), result);
    }

    public Boolean getBooleanValue(final String... expressions)
    {
        return ELUtils.getBooleanValue(getELContext(), selectFirstNonNull(expressions));
    }

    public BuildConfiguratorPluginConfiguration getConfiguration()
    {
        return getRequest().getPluginConfiguration();
    }

    public <T extends BuildConfiguratorPluginConfiguration> T getConfiguration(
            final Class<T> configurationType)
    {
        if (configurationType.isAssignableFrom(getConfiguration().getClass()))
        {
            @SuppressWarnings("unchecked")
            final T configuration = (T) getConfiguration();

            return configuration;
        }

        return null;
    }

    public ELContext getELContext()
    {
        return elContext;
    }

    public MavenSession getMavenSession()
    {
        return mavenSession;
    }

    public MavenProject getProject()
    {
        return getRequest().getProject();
    }

    public Configuration getProperties()
    {
        return properties;
    }

    public ConfigureRequest getRequest()
    {
        return request;
    }

    public <T> T getResult(final Class<T> resultType)
    {
        @SuppressWarnings("unchecked")
        final T result = (T) results.get(resultType.getName());

        return result;
    }

    public Map<String, Object> getResults()
    {
        return results;
    }

    public String getStringValue(final String... expressions)
    {
        return ELUtils.getStringValue(getELContext(), selectFirstNonNull(expressions));
    }

    protected <T> T selectFirstNonNull(final T... values)
    {
        for (final T value : values)
        {
            if (value != null)
            {
                return value;
            }
        }

        return null;
    }
}
