/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configurators;

import java.util.Arrays;
import java.util.List;

import javax.el.ELContext;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.maven.model.Dependency;

import com.github.htfv.maven.plugins.buildconfigurator.core.ParameterValueInvalidException;
import com.github.htfv.maven.plugins.buildconfigurator.core.RequiredParameterNotSpecifiedException;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.DependencyConfiguration;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils.MergeHelper;

/**
 * {@link MergeHelper} which assists in dependency merging.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class DependencyMergeHelper implements MergeHelper<Dependency, DependencyConfiguration>
{
    public static final String[] ADD_ACTIONS =
    {
        DependencyConfiguration.ACTION_ADD,
        DependencyConfiguration.ACTION_ADD_UPDATE
    };

    public static final String DEFAULT_ACTION = DependencyConfiguration.ACTION_ADD_UPDATE;
    public static final String DEFAULT_TYPE = DependencyConfiguration.DEFAULT_TYPE;

    public static final String[] UPDATE_ACTIONS =
    {
        DependencyConfiguration.ACTION_ADD_UPDATE,
        DependencyConfiguration.ACTION_UPDATE
    };

    public static final String[] VALID_ACTIONS =
    {
        DependencyConfiguration.ACTION_ADD,
        DependencyConfiguration.ACTION_ADD_UPDATE,
        DependencyConfiguration.ACTION_SKIP,
        DependencyConfiguration.ACTION_UPDATE
    };

    private final ConfigurationContext ctx;
    private final String root;

    public DependencyMergeHelper(final ConfigurationContext ctx)
    {
        this(ctx, "");
    }

    /**
     * Creates a new instance of {@link DependencyMergeHelper}.
     *
     * @param elContext
     *            {@link ELContext} used to evaluate EL expressions.
     */
    public DependencyMergeHelper(final ConfigurationContext ctx, final String root)
    {
        this.ctx  = ctx;
        this.root = root;
    }

    /**
     * Converts the given source {@link DependencyConfiguration} object into a new target
     * {@link Dependency} object.
     *
     * @param dependency
     *            source {@link DependencyConfiguration} object.
     *
     * @return a new target {@link Dependency} object or {@code null} if
     *         {@link DependencyConfiguration#getAction} is {@code skip} or not {@code add}
     *         or {@code add-update}.
     */
    @Override
    public Dependency add(final DependencyConfiguration dependency)
    {
        final String action = getAction(dependency);

        if (!getAddActions().contains(action))
        {
            return null;
        }

        final Dependency projectDependency = new Dependency();

        projectDependency.setGroupId(getGroupId(dependency));
        projectDependency.setArtifactId(getArtifactId(dependency));
        projectDependency.setType(getType(dependency));
        projectDependency.setClassifier(ctx.getStringValue(dependency.getClassifier()));

        return updateInternal(projectDependency, dependency);
    }

    /**
     * Compares the given target {@link Dependency} object with the given source
     * {@link DependencyConfiguration} object using {@code groupId, artifactId, type,} and
     * {@code classifier}.
     *
     * @param projectDependency
     *            target {@link Dependency} object.
     * @param dependency
     *            source {@link DependencyConfiguration} object.
     *
     * @return {@code true} if the given target and source objects may be
     *         considered equal.
     */
    @Override
    public boolean equals(final Dependency projectDependency,
            final DependencyConfiguration dependency)
    {
        final String groupId = getGroupId(dependency);

        if (ObjectUtils.notEqual(projectDependency.getGroupId(), groupId))
        {
            return false;
        }

        final String artifactId = getArtifactId(dependency);

        if (ObjectUtils.notEqual(projectDependency.getArtifactId(), artifactId))
        {
            return false;
        }

        final String type = getType(dependency);

        if (ObjectUtils.notEqual(projectDependency.getType(), type))
        {
            return false;
        }

        final String classifier = ctx.getStringValue(dependency.getClassifier());

        if (ObjectUtils.notEqual(projectDependency.getClassifier(), classifier))
        {
            return false;
        }

        return true;
    }

    protected String getAction(final DependencyConfiguration dependency)
    {
        final String action = ctx.getStringValue(
                dependency.getAction(),
                getDefaultAction());

        if (!getValidActions().contains(action))
        {
            throw new ParameterValueInvalidException(root + "dependencies/dependency/action");
        }

        return action;
    }

    protected List<String> getAddActions()
    {
        return Arrays.asList(ADD_ACTIONS);
    }

    protected String getArtifactId(final DependencyConfiguration dependency)
    {
        final String artifactId = ctx.getStringValue(dependency.getArtifactId());

        if (StringUtils.isEmpty(artifactId))
        {
            throw new RequiredParameterNotSpecifiedException(
                    root + "dependencies/dependency/artifactId");
        }

        return artifactId;
    }

    protected String getDefaultAction()
    {
        return DEFAULT_ACTION;
    }

    protected String getDefaultType()
    {
        return DEFAULT_TYPE;
    }

    protected String getGroupId(final DependencyConfiguration dependency)
    {
        final String groupId = ctx.getStringValue(dependency.getGroupId());

        if (StringUtils.isEmpty(groupId))
        {
            throw new RequiredParameterNotSpecifiedException(
                    root + "dependencies/dependency/groupId");
        }

        return groupId;
    }

    protected String getType(final DependencyConfiguration dependency)
    {
        return ctx.getStringValue(
                dependency.getType(),
                getDefaultType());
    }

    protected List<String> getUpdateActions()
    {
        return Arrays.asList(UPDATE_ACTIONS);
    }

    protected List<String> getValidActions()
    {
        return Arrays.asList(VALID_ACTIONS);
    }

    /**
     * Updates the given target {@link Dependency} object with the given source
     * {@link DependencyConfiguration} object.
     *
     * @param projectDependency
     *            target {@link Dependency} object.
     * @param dependency
     *            source {@link DependencyConfiguration} object.
     *
     * @return updated target {@link Dependency} object or {@code null} if
     *         {@link DependencyConfiguration#getAction} is {@code skip} or not
     *         {@code add-update} or {@code update}.
     */
    @Override
    public Dependency update(final Dependency projectDependency, final DependencyConfiguration dependency)
    {
        final String action = getAction(dependency);

        if (!getUpdateActions().contains(action))
        {
            return projectDependency;
        }

        return updateInternal(projectDependency, dependency);
    }

    /**
     * Updates all non-key properties of the given {@link Dependency} object and
     * merges contained dependency exclusions.
     *
     * @param projectDependency
     *            target {@link Dependency} object.
     * @param dependency
     *            source {@link DependencyConfiguration} object.
     *
     * @return updated target {@link Dependency} object.
     */
    public Dependency updateInternal(final Dependency projectDependency,
            final DependencyConfiguration dependency)
    {
        if (StringUtils.isNotEmpty(dependency.getVersion()))
        {
            projectDependency.setVersion(ctx.getStringValue(dependency.getVersion()));
        }

        if (StringUtils.isNotEmpty(dependency.getScope()))
        {
            projectDependency.setScope(ctx.getStringValue(dependency.getScope()));
        }

        if (StringUtils.isNotEmpty(dependency.getSystemPath()))
        {
            projectDependency.setSystemPath(ctx.getStringValue(dependency.getSystemPath()));
        }

        if (StringUtils.isNotEmpty(dependency.getOptional()))
        {
            projectDependency.setOptional(ctx.getBooleanValue(dependency.getOptional()));
        }

        if (dependency.getExclusions() != null)
        {
            MergeUtils.mergeCollections(
                    projectDependency.getExclusions(),
                    dependency.getExclusions(),
                    new ExclusionMergeHelper(ctx));
        }

        return projectDependency;
    }
}
