/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configurators;

import java.util.Arrays;
import java.util.List;

import javax.el.ELContext;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.maven.model.Exclusion;

import com.github.htfv.maven.plugins.buildconfigurator.core.ParameterValueInvalidException;
import com.github.htfv.maven.plugins.buildconfigurator.core.RequiredParameterNotSpecifiedException;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.ExclusionConfiguration;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils.MergeHelper;

/**
 * {@link MergeHelper} which assists in dependency exclusion merging.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class ExclusionMergeHelper implements MergeHelper<Exclusion, ExclusionConfiguration>
{
    private static final String DEFAULT_ACTION = ExclusionConfiguration.ACTION_ADD;

    private static final String[] VALID_ACTIONS =
    {
        ExclusionConfiguration.ACTION_ADD,
        ExclusionConfiguration.ACTION_SKIP
    };

    protected final ConfigurationContext ctx;

    protected final String root;

    public ExclusionMergeHelper(final ConfigurationContext ctx)
    {
        this(ctx, "");
    }

    /**
     * Creates a new instance of {@link ExclusionMergeHelper}.
     *
     * @param elContext
     *            {@link ELContext} used to evaluate EL expressions.
     */
    public ExclusionMergeHelper(final ConfigurationContext ctx, final String root)
    {
        this.ctx  = ctx;
        this.root = root;
    }

    /**
     * Converts the given source {@link ExclusionConfiguration} object into a new target
     * {@link Exclusion} object.
     *
     * @param exclusion
     *            source {@link ExclusionConfiguration} object.
     *
     * @return a new target {@link Exclusion} object or {@code null} if
     *         {@link ExclusionConfiguration#getAction} is {@code skip}.
     */
    @Override
    public Exclusion add(final ExclusionConfiguration exclusion)
    {
        final String action = getAction(exclusion);

        if (ExclusionConfiguration.ACTION_SKIP.equals(action))
        {
            return null;
        }

        final Exclusion projectExclusion = new Exclusion();

        projectExclusion.setGroupId(getGroupId(exclusion));
        projectExclusion.setArtifactId(getArtifactId(exclusion));

        return projectExclusion;
    }

    /**
     * Compares the given target {@link Exclusion} object with the given source
     * {@link ExclusionConfiguration} object using {@code groupId} and {@code artifactId}.
     *
     * @param projectExclusion
     *            target {@link Exclusion} object.
     * @param exclusion
     *            source {@link ExclusionConfiguration} object.
     *
     * @return {@code true} if the given target and source objects may be
     *         considered equal.
     */
    @Override
    public boolean equals(final Exclusion projectExclusion, final ExclusionConfiguration exclusion)
    {
        final String groupId = getGroupId(exclusion);

        if (ObjectUtils.notEqual(projectExclusion.getGroupId(), groupId))
        {
            return false;
        }

        final String artifactId = getArtifactId(exclusion);

        if (ObjectUtils.notEqual(projectExclusion.getArtifactId(), artifactId))
        {
            return false;
        }

        return true;
    }

    protected String getAction(final ExclusionConfiguration exclusion)
    {
        final String action = ctx.getStringValue(
                exclusion.getAction(),
                getDefaultAction());

        if (!getValidActions().contains(action))
        {
            throw new ParameterValueInvalidException(
                    root + "dependencies/dependency/exclusions/exclusion/action");
        }

        return action;
    }

    protected String getArtifactId(final ExclusionConfiguration exclusion)
    {
        final String artifactId = ctx.getStringValue(exclusion.getArtifactId());

        if (StringUtils.isEmpty(artifactId))
        {
            throw new RequiredParameterNotSpecifiedException(
                    root + "dependencies/dependency/exclusions/exclusion/artifactId");
        }

        return artifactId;
    }

    protected String getDefaultAction()
    {
        return DEFAULT_ACTION;
    }

    protected String getGroupId(final ExclusionConfiguration exclusion)
    {
        final String groupId = ctx.getStringValue(exclusion.getGroupId());

        if (StringUtils.isEmpty(groupId))
        {
            throw new RequiredParameterNotSpecifiedException(
                    root + "dependencies/dependency/exclusions/exclusion/groupId");
        }

        return groupId;
    }

    protected List<String> getValidActions()
    {
        return Arrays.asList(VALID_ACTIONS);
    }

    /**
     * Updates the given target {@link Exclusion} object with the given source
     * {@link ExclusionConfiguration} object.
     *
     * @param projectExclusion
     *            target {@link Exclusion} object.
     * @param exclusion
     *            source {@link ExclusionConfiguration} object.
     *
     * @return updated target {@link Exclusion} object.
     */
    @Override
    public Exclusion update(final Exclusion projectExclusion,
            final ExclusionConfiguration exclusion)
    {
        return projectExclusion;
    }
}
