/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configurators;

import java.util.Arrays;
import java.util.List;

import javax.el.ELContext;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.maven.model.PluginExecution;

import com.github.htfv.maven.plugins.buildconfigurator.core.ParameterValueInvalidException;
import com.github.htfv.maven.plugins.buildconfigurator.core.StringMergeHelper;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.ExecutionConfiguration;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils.MergeHelper;

/**
 * {@link MergeHelper} which assists in plugin execution merging.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class PluginExecutionMergeHelper implements MergeHelper<PluginExecution, ExecutionConfiguration>
{
    public static final String DEFAULT_ACTION = ExecutionConfiguration.ACTION_ADD_UPDATE;

    public static final String DEFAULT_ID = "default";

    public static final String[] VALID_ACTIONS =
    {
        ExecutionConfiguration.ACTION_ADD,
        ExecutionConfiguration.ACTION_ADD_UPDATE,
        ExecutionConfiguration.ACTION_SKIP,
        ExecutionConfiguration.ACTION_UPDATE
    };

    private final ConfigurationContext ctx;

    /**
     * Creates a new instance of {@link PluginExecutionMergeHelper}.
     *
     * @param elContext
     *            {@link ELContext} used to evaluate EL expressions.
     */
    public PluginExecutionMergeHelper(final ConfigurationContext ctx)
    {
        this.ctx = ctx;
    }

    /**
     * Converts the given source {@link ExecutionConfiguration} object into a new target
     * {@link PluginExecution} object.
     *
     * @param execution
     *            source {@link ExecutionConfiguration} object.
     *
     * @return a new target {@link PluginExecution} object or {@code null} if
     *         {@link ExecutionConfiguration#getAction} is {@code skip} or not {@code add}
     *         or {@code add-update}.
     */
    @Override
    public PluginExecution add(final ExecutionConfiguration execution)
    {
        final String action = getAction(execution);

        if (ExecutionConfiguration.ACTION_SKIP.equals(action)
                || !ExecutionConfiguration.ACTION_ADD.equals(action)
                        && !ExecutionConfiguration.ACTION_ADD_UPDATE.equals(action))
        {
            return null;
        }

        final PluginExecution projectExecution = new PluginExecution();

        projectExecution.setId(ctx.getStringValue(execution.getId()));

        return updateInternal(projectExecution, execution);
    }

    /**
     * Compares the given target {@link PluginExecution} object with the given
     * source {@link ExecutionConfiguration} object using {@code id}.
     *
     * @param projectExecution
     *            target {@link PluginExecution} object.
     * @param execution
     *            source {@link ExecutionConfiguration} object.
     *
     * @return {@code true} if the given target and source objects may be
     *         considered equal.
     */
    @Override
    public boolean equals(final PluginExecution projectExecution, final ExecutionConfiguration execution)
    {
        final String id = ctx.getStringValue(
                execution.getId(),
                getDefaultId());

        if (ObjectUtils.notEqual(projectExecution.getId(), id))
        {
            return false;
        }

        return true;
    }

    protected String getAction(final ExecutionConfiguration execution)
    {
        final String action = ctx.getStringValue(
                execution.getAction(),
                getDefaultAction());

        if (!getValidActions().contains(action))
        {
            throw new ParameterValueInvalidException(
                    "build/plugins/plugin/executions/execution/action");
        }

        return action;
    }

    protected String getDefaultAction()
    {
        return DEFAULT_ACTION;
    }

    protected String getDefaultId()
    {
        return DEFAULT_ID;
    }

    protected List<String> getValidActions()
    {
        return Arrays.asList(VALID_ACTIONS);
    }

    /**
     * Updates the given target {@link PluginExecution} object with the given
     * source {@link ExecutionConfiguration} object.
     *
     * @param projectExecution
     *            target {@link PluginExecution} object.
     * @param execution
     *            source {@link ExecutionConfiguration} object.
     *
     * @return updated target {@link PluginExecution} object or {@code null} if
     *         {@link ExecutionConfiguration#getAction} is {@code skip} or not
     *         {@code add-update} or {@code update}.
     */
    @Override
    public PluginExecution update(final PluginExecution projectExecution,
            final ExecutionConfiguration execution)
    {
        final String action = getAction(execution);

        if (ExecutionConfiguration.ACTION_SKIP.equals(action)
                || !ExecutionConfiguration.ACTION_ADD_UPDATE.equals(action)
                        && !ExecutionConfiguration.ACTION_UPDATE.equals(action))
        {
            return projectExecution;
        }

        return updateInternal(projectExecution, execution);
    }

    /**
     * Updates all non-key properties of the given {@link PluginExecution}
     * object and merges contained configuration and goals.
     *
     * @param projectExecution
     *            target {@link PluginExecution} object.
     * @param execution
     *            source {@link ExecutionConfiguration} object.
     *
     * @return updated target {@link PluginExecution} object.
     */
    public PluginExecution updateInternal(final PluginExecution projectExecution,
            final ExecutionConfiguration execution)
    {
        projectExecution.setConfiguration(
                MergeUtils.mergeConfiguration(
                        projectExecution.getConfiguration(),
                        execution.getConfiguration(),
                        ctx.getELContext()));

        if (execution.getGoals() != null)
        {
            MergeUtils.mergeCollections(
                    projectExecution.getGoals(),
                    execution.getGoals(),
                    new StringMergeHelper(ctx.getELContext()));
        }

        if (StringUtils.isNotEmpty(execution.getPhase()))
        {
            projectExecution.setPhase(ctx.getStringValue(execution.getPhase()));
        }

        return projectExecution;
    }
}
