/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configurators;

import java.util.Arrays;
import java.util.List;

import javax.el.ELContext;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.maven.model.Plugin;

import com.github.htfv.maven.plugins.buildconfigurator.core.ParameterValueInvalidException;
import com.github.htfv.maven.plugins.buildconfigurator.core.RequiredParameterNotSpecifiedException;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.PluginConfiguration;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils;
import com.github.htfv.maven.plugins.buildconfigurator.core.utils.MergeUtils.MergeHelper;

/**
 * {@link MergeHelper} which assists in plugin merging.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public class PluginMergeHelper implements MergeHelper<Plugin, PluginConfiguration>
{
    public static final String[] ADD_ACTIONS =
    {
        PluginConfiguration.ACTION_ADD,
        PluginConfiguration.ACTION_ADD_UPDATE
    };

    public static final String DEFAULT_ACTION = PluginConfiguration.ACTION_ADD_UPDATE;

    public static final String[] UPDATE_ACTIONS =
    {
        PluginConfiguration.ACTION_ADD_UPDATE,
        PluginConfiguration.ACTION_UPDATE
    };

    public static final String[] VALID_ACTIONS =
    {
        PluginConfiguration.ACTION_ADD,
        PluginConfiguration.ACTION_ADD_UPDATE,
        PluginConfiguration.ACTION_SKIP,
        PluginConfiguration.ACTION_UPDATE
    };

    private final ConfigurationContext ctx;

    /**
     * Creates a new instance of {@link PluginMergeHelper}.
     *
     * @param elContext
     *            {@link ELContext} used to evaluate EL expressions.
     */
    public PluginMergeHelper(final ConfigurationContext ctx)
    {
        this.ctx = ctx;
    }

    /**
     * Converts the given source {@link PluginConfiguration} object into a new target
     * {@link Plugin} object.
     *
     * @param plugin
     *            source {@link PluginConfiguration} object.
     *
     * @return a new target {@link Plugin} object or {@code null} if
     *         {@link PluginConfiguration#getAction} is {@code skip} or not {@code add} or
     *         {@code add-update}.
     */
    @Override
    public Plugin add(final PluginConfiguration plugin)
    {
        final String action = getAction(plugin);

        if (!getAddActions().contains(action))
        {
            return null;
        }

        final Plugin projectPlugin = new Plugin();

        projectPlugin.setGroupId(ctx.getStringValue(plugin.getGroupId()));
        projectPlugin.setArtifactId(ctx.getStringValue(plugin.getArtifactId()));

        return updateInternal(projectPlugin, plugin);
    }

    /**
     * Compares the given target {@link Plugin} object with the given source
     * {@link PluginConfiguration} object using {@code groupId} and {@code artifactId}.
     *
     * @param projectPlugin
     *            target {@link Plugin} object.
     * @param plugin
     *            source {@link PluginConfiguration} object.
     *
     * @return {@code true} if the given target and source objects may be
     *         considered equal.
     */
    @Override
    public boolean equals(final Plugin projectPlugin, final PluginConfiguration plugin)
    {
        final String groupId = ctx.getStringValue(plugin.getGroupId());

        if (ObjectUtils.notEqual(projectPlugin.getGroupId(), groupId))
        {
            return false;
        }

        final String artifactId = ctx.getStringValue(plugin.getArtifactId());

        if (ObjectUtils.notEqual(projectPlugin.getArtifactId(), artifactId))
        {
            return false;
        }

        return true;
    }

    protected String getAction(final PluginConfiguration plugin)
    {
        final String action = ctx.getStringValue(
                plugin.getAction(),
                getDefaultPluginAction());

        if (!getValidPluginActions().contains(action))
        {
            throw new ParameterValueInvalidException("build/plugins/plugin/action");
        }

        return action;
    }

    protected List<String> getAddActions()
    {
        return Arrays.asList(ADD_ACTIONS);
    }

    protected String getDefaultPluginAction()
    {
        return DEFAULT_ACTION;
    }

    protected List<String> getUpdateActions()
    {
        return Arrays.asList(UPDATE_ACTIONS);
    }

    protected List<String> getValidPluginActions()
    {
        return Arrays.asList(VALID_ACTIONS);
    }

    /**
     * Updates the given target {@link Plugin} object with the given source
     * {@link PluginConfiguration} object.
     *
     * @param projectPlugin
     *            target {@link Plugin} object.
     * @param plugin
     *            source {@link PluginConfiguration} object.
     *
     * @return updated target {@link Plugin} object or {@code null} if
     *         {@link PluginConfiguration#getAction} is {@code skip} or not
     *         {@code add-update} or {@code update}.
     */
    @Override
    public Plugin update(final Plugin projectPlugin, final PluginConfiguration plugin)
    {
        final String action = getAction(plugin);

        if (!getUpdateActions().contains(action))
        {
            return projectPlugin;
        }

        return updateInternal(projectPlugin, plugin);
    }

    /**
     * Updates all non-key properties of the given {@link Plugin} object and
     * merges contained configuration, dependencies, and executions.
     *
     * @param projectPlugin
     *            target {@link Plugin} object.
     * @param plugin
     *            source {@link PluginConfiguration} object.
     *
     * @return updated target {@link Plugin} object.
     */
    private Plugin updateInternal(final Plugin projectPlugin, final PluginConfiguration plugin)
    {
        if (StringUtils.isEmpty(plugin.getGroupId()))
        {
            throw new RequiredParameterNotSpecifiedException("build/plugins/plugin/groupId");
        }

        if (StringUtils.isEmpty(plugin.getArtifactId()))
        {
            throw new RequiredParameterNotSpecifiedException("build/plugins/plugin/artifactId");
        }

        if (StringUtils.isNotEmpty(plugin.getVersion()))
        {
            projectPlugin.setVersion(ctx.getStringValue(plugin.getVersion()));
        }

        projectPlugin.setConfiguration(
                MergeUtils.mergeConfiguration(
                        projectPlugin.getConfiguration(),
                        plugin.getConfiguration(),
                        ctx.getELContext()));

        if (plugin.getDependencies() != null)
        {
            MergeUtils.mergeCollections(
                    projectPlugin.getDependencies(),
                    plugin.getDependencies(),
                    new DependencyMergeHelper(ctx));
        }

        if (plugin.getExecutions() != null)
        {
            if (MergeUtils.mergeCollections(
                    projectPlugin.getExecutions(),
                    plugin.getExecutions(),
                    new PluginExecutionMergeHelper(ctx)))
            {
                projectPlugin.flushExecutionMap();
            }
        }

        return projectPlugin;
    }
}
