/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.configurators;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.lang3.StringUtils;
import org.apache.maven.execution.ExecutionEvent;
import org.apache.maven.execution.ExecutionListener;
import org.apache.maven.execution.MavenExecutionRequest;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.project.MavenProject;

import com.github.htfv.maven.plugins.buildconfigurator.core.DelegatingExecutionListener;
import com.github.htfv.maven.plugins.buildconfigurator.core.ModelInterpolator;
import com.github.htfv.maven.plugins.buildconfigurator.core.ParameterValueInvalidException;
import com.github.htfv.maven.plugins.buildconfigurator.core.PropertyFileMissingException;
import com.github.htfv.maven.plugins.buildconfigurator.core.RequiredParameterNotSpecifiedException;
import com.github.htfv.maven.plugins.buildconfigurator.core.configuration.PropertyFileConfiguration;

public class PropertyFileConfigurator implements Configurator
{
    public static class Result
    {
        public static class Builder
        {
            private final Map<String, String> newProperties = new HashMap<String, String>();

            public Result build()
            {
                return new Result(this);
            }

            public Map<String, String> newProperties()
            {
                return newProperties;
            }

            public Builder newProperty(final String key, final String value)
            {
                newProperties.put(key, value);
                return this;
            }
        }

        private final Map<String, String> newProperties;

        protected Result(final Builder builder)
        {
            this.newProperties = Collections.unmodifiableMap(builder.newProperties);
        }

        public Map<String, String> getNewProperties()
        {
            return newProperties;
        }
    }

    public static final String DEFAULT_ACTION = PropertyFileConfiguration.ACTION_LOAD;

    public static final String DEFAULT_IGNORE_MISSING_PROPERTY_FILES = "false";

    public static final String DEFAULT_PROPERTY_FILE_ENCODING = "ISO-8859-1";

    public static final String[] VALID_ACTIONS =
    {
        PropertyFileConfiguration.ACTION_LOAD,
        PropertyFileConfiguration.ACTION_SKIP
    };

    @Override
    public void configure(final ConfigurationContext ctx) throws Exception
    {
        final Result.Builder resultBuilder = new Result.Builder();

        try
        {
            final List<PropertyFileConfiguration> propertyFiles =
                    ctx.getConfiguration().getPropertyFiles();

            if (propertyFiles == null)
            {
                return;
            }

            for (final PropertyFileConfiguration propertyFile : propertyFiles)
            {
                loadPropertyFile(ctx, propertyFile);
            }

            updateProjectProperties(ctx, resultBuilder);
            reinterpolateProjectModel(ctx, resultBuilder);
        }
        finally
        {
            ctx.addResult(resultBuilder.build());
        }
    }

    protected String getAction(final ConfigurationContext ctx,
            final PropertyFileConfiguration propertyFile)
    {
        final String action = ctx.getStringValue(
                propertyFile.getAction(),
                getDefaultAction());

        if (!getValidActions().contains(action))
        {
            throw new ParameterValueInvalidException("propertyFiles/propertyFile/action");
        }

        return action;
    }

    protected String getDefaultAction()
    {
        return DEFAULT_ACTION;
    }

    protected String getDefaultIgnoreMissingPropertyFiles()
    {
        return DEFAULT_IGNORE_MISSING_PROPERTY_FILES;
    }

    protected String getDefaultPropertyFileEncoding()
    {
        return DEFAULT_PROPERTY_FILE_ENCODING;
    }

    protected File getFile(final ConfigurationContext ctx,
            final PropertyFileConfiguration propertyFile)
    {
        if (StringUtils.isEmpty(propertyFile.getFile()))
        {
            throw new RequiredParameterNotSpecifiedException("propertyFiles/propertyFile/file");
        }

        File file = new File(ctx.getStringValue(propertyFile.getFile()));

        if (!file.isAbsolute())
        {
            file = new File(ctx.getProject().getBasedir(), file.getPath());
        }

        return file;
    }

    protected List<String> getValidActions()
    {
        return Arrays.asList(VALID_ACTIONS);
    }

    protected void loadPropertyFile(final ConfigurationContext ctx,
            final PropertyFileConfiguration propertyFile) throws ConfigurationException, IOException
    {
        BufferedReader reader = null;

        try
        {
            final String action = getAction(ctx, propertyFile);

            if (PropertyFileConfiguration.ACTION_SKIP.equals(action))
            {
                return;
            }

            //
            // Check to see whether the file exists.
            //

            final File file = getFile(ctx, propertyFile);

            if (!file.exists())
            {
                //
                // Throw an error if missing file is not allowed.
                //

                final Boolean ignoreMissing = ctx.getBooleanValue(
                        propertyFile.getIgnoreMissing(),
                        ctx.getConfiguration().getIgnoreMissingPropertyFiles(),
                        getDefaultIgnoreMissingPropertyFiles());

                if (ignoreMissing)
                {
                    return;
                }
                else
                {
                    throw new PropertyFileMissingException(file);
                }
            }

            //
            // Open file reader with appropriate encoding.
            //

            final String encoding = ctx.getStringValue(
                    propertyFile.getEncoding(),
                    ctx.getConfiguration().getDefaultPropertyFilesEncoding(),
                    getDefaultPropertyFileEncoding());

            reader = new BufferedReader(new InputStreamReader(
                    new FileInputStream(file), encoding));

            //
            // Load the properties.
            //

            final PropertiesConfiguration properties = new PropertiesConfiguration();

            properties.setDelimiterParsingDisabled(true);
            properties.load(reader);

            //
            // Add loaded properties to the context.
            //

            ctx.addProperties(properties);
        }
        finally
        {
            if (reader != null)
            {
                reader.close();
            }
        }
    }

    /**
     * Reinterpolates all elements of the project model.
     *
     * @param ctx
     *            parameters for configuring a project.
     */
    private void reinterpolateProjectModel(final ConfigurationContext ctx,
            final Result.Builder resultBuilder)
    {
        //
        // Maven interpolates expressions when it loads the model. Since the
        // properties are loaded after the model, we have to interpolate
        // expressions again.
        //
        // It is actually not required for mojo parameters, because parameters
        // are interpolated before mojo is executed. However, it can be a
        // problem if mojo accesses model, like maven-resources-plugin accessing
        // project.build.resources.
        //
        // We take a conservative approach and only replace properties which
        // were loaded by the build configurator.
        //

        final ModelInterpolator interpolator =
                new ModelInterpolator(resultBuilder.newProperties());

        interpolator.interpolateModel(ctx.getProject().getModel());

        //
        // Maven interpolates mojo parameters before execution, but it skips
        // parameters of the PlexusConfiguration type. Updating the model now
        // has no effect, because execution plan is already calculated and mojo
        // configuration is copied to mojo execution. We cannot get access to
        // mojo execution, so we register a listener which will be called before
        // mojo is executed.
        //

        if (!ctx.getRequest().isConnectorRequest())
        {
            final MavenSession          mavenSession      = ctx.getMavenSession();
            final MavenExecutionRequest executionRequest  = mavenSession.getRequest();
            final ExecutionListener     executionListener = executionRequest.getExecutionListener();

            executionRequest.setExecutionListener(
                    new DelegatingExecutionListener(executionListener)
                    {
                        @Override
                        public void mojoStarted(final ExecutionEvent event)
                        {
                            interpolator.interpolateDOM(
                                    event.getMojoExecution().getConfiguration());

                            super.mojoStarted(event);
                        }

                        @Override
                        public void projectFailed(final ExecutionEvent event)
                        {
                            event.getSession().getRequest().setExecutionListener(getDelegate());
                            super.projectFailed(event);
                        }

                        @Override
                        public void projectSucceeded(final ExecutionEvent event)
                        {
                            event.getSession().getRequest().setExecutionListener(getDelegate());
                            super.projectSucceeded(event);
                        }
                    });
        }
    }

    private void updateProjectProperties(final ConfigurationContext ctx,
            final Result.Builder resultBuilder)
    {
        final Configuration properties        = ctx.getProperties();
        final MavenProject  project           = ctx.getProject();
        final Properties    projectProperties = project.getProperties();

        @SuppressWarnings("unchecked")
        final Iterator<String> keys = properties.getKeys();

        while (keys.hasNext())
        {
            final String key = keys.next();

            if (project.getProjectBuildingRequest().getUserProperties().containsKey(key)
                    || projectProperties.containsKey(key))
            {
                continue;
            }

            final String value = ctx.getStringValue(properties.getString(key));

            projectProperties.setProperty(key, value);
            resultBuilder.newProperty(key, value);
        }
    }
}
