/*
 * Copyright 2012 htfv (Aliaksei Lahachou)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.htfv.maven.plugins.buildconfigurator.core.utils;

import java.io.StringReader;
import java.util.Arrays;
import java.util.List;

import javax.el.ELContext;

import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import org.codehaus.plexus.util.xml.Xpp3DomBuilder;

/**
 * Contains helper methods for merging collections and configurations.
 *
 * @author htfv (Aliaksei Lahachou)
 */
public abstract class MergeUtils
{
    /**
     * Common interface of the helpers which assist merging of collections.
     *
     * @author htfv (Aliaksei Lahachou)
     */
    public interface MergeHelper<Target, Source>
    {
        /**
         * Converts the given source item into a new target item which can be
         * inserted into the target collection.
         *
         * @param sourceItem
         *            source item.
         *
         * @return a new instance of the target item or {@code null} if the
         *         source item shall not be added to the target collection.
         */
        Target add(Source sourceItem);

        /**
         * Compares the given target item with the given source item.
         *
         * @param targetItem
         *            target item.
         * @param sourceItem
         *            source item.
         *
         * @return {@code true} if the given target and source items may be
         *         considered equal.
         */
        boolean equals(Target targetItem, Source sourceItem);

        /**
         * Merges the given source item into the given target item.
         *
         * @param targetItem
         *            item into which the source item is merged.
         * @param sourceItem
         *            item which is merged into the target item.
         *
         * @return target item; this method may return a new instance.
         */
        Target update(Target targetItem, Source sourceItem);
    }

    /**
     * Merges two collections retaining the relative order of the elements. The
     * order of elements in the target collection dominates over the order in
     * the source collection.
     * <p>
     * <table>
     * <tr>
     * <th>Target</th>
     * <th>Source</th>
     * <th>Result</th>
     * </tr>
     * <tr>
     * <td>{@code A}</td>
     * <td>{@code A}</td>
     * <td>{@code A}</td>
     * </tr>
     * <tr>
     * <td>{@code B}</td>
     * <td></td>
     * <td>{@code B}</td>
     * </tr>
     * <tr>
     * <td></td>
     * <td>{@code C}</td>
     * <td>{@code C}</td>
     * </tr>
     * <tr>
     * <td>{@code D}</td>
     * <td>{@code D}</td>
     * <td>{@code D}</td>
     * </tr>
     * <tr>
     * <td></td>
     * <td></td>
     * <td>{@code F}</td>
     * <td>F comes before X in the source collection.
     * </tr>
     * <tr>
     * <td>{@code X}</td>
     * <td></td>
     * <td>{@code X}</td>
     * </tr>
     * <tr>
     * <td>{@code E}</td>
     * <td></td>
     * <td>{@code E}</td>
     * </tr>
     * <tr>
     * <td>{@code Y}</td>
     * <td>{@code Y}</td>
     * <td>{@code Y}</td>
     * </tr>
     * <tr>
     * <td></td>
     * <td>{@code F}</td>
     * <td></td>
     * </tr>
     * <tr>
     * <td></td>
     * <td>{@code X}</td>
     * <td></td>
     * </tr>
     * </table>
     *
     * @param <Target>
     *            the type of the items in the target collection.
     * @param <Source>
     *            the type of the items in the source collection.
     *
     * @param targetCollection
     *            collection into which the source items are merged.
     * @param sourceCollection
     *            collection whose items are merged into target collection.
     * @param mergeHelper
     *            helper used to compare, convert, and merge source items to
     *            target items.
     *
     * @return {@code true} if target collection was modified (new items added
     *         or existing items updated).
     */
    public static <Target, Source> boolean mergeCollections(final List<Target> targetCollection,
            final List<Source> sourceCollection, final MergeHelper<Target, Source> mergeHelper)
    {
        boolean targetCollectionUpdated = false;

        //
        // Indices of the source elements in the target collection.
        //

        final int[] indices = new int[sourceCollection.size()];
        Arrays.fill(indices, -1);

        //
        // Update elements which exist in both collections. At the same time,
        // save the indices of the source elements in the target collection.
        //

        for (int t = 0, ts = targetCollection.size(); t < ts; t += 1)
        {
            final Target targetElement = targetCollection.get(t);

            for (int s = 0, ss = sourceCollection.size(); s < ss; s += 1)
            {
                final Source sourceElement = sourceCollection.get(s);

                if (mergeHelper.equals(targetElement, sourceElement))
                {
                    targetCollection.set(t, mergeHelper.update(targetElement, sourceElement));
                    targetCollectionUpdated = true;
                    indices[s] = t;
                    break;
                }
            }
        }

        //
        // Now, when we know which source elements exist in the target
        // collection and their indices, we may add new source elements.
        //

        int insertAt = targetCollection.size();

        for (int s = sourceCollection.size() - 1; s >= 0; s -= 1)
        {
            final int i = indices[s];

            if (i < 0)
            {
                final Target targetEl = mergeHelper.add(sourceCollection.get(s));

                if (targetEl != null)
                {
                    targetCollection.add(insertAt, targetEl);
                    targetCollectionUpdated = true;
                }
            }
            else
            {
                if (i < insertAt)
                {
                    insertAt = i;
                }
            }
        }

        return targetCollectionUpdated;
    }

    /**
     * Merges two plugin configurations. The source configuration dominates over
     * the target configuration, i.e. values from the source configuration
     * override values in the target configuration.
     *
     * @param targetConfiguration
     *            configuration into which the source configuration values are
     *            merged; the target configuration is usually obtained by
     *            calling {@link org.apache.maven.model.Plugin#getConfiguration}.
     * @param sourceConfiguration
     *            configuration which is merged into the target configuration.
     * @param elContext
     *            {@link ELContext} used to evaluate EL expressions.
     *
     * @return merged configuration; this can be a new instance, or modified old
     *         one.
     */
    public static Object mergeConfiguration(final Object targetConfiguration,
            final PlexusConfiguration sourceConfiguration, final ELContext elContext)
    {
        try
        {
            if (sourceConfiguration == null)
            {
                return targetConfiguration;
            }

            final Xpp3Dom sourceConfigurationDOM = Xpp3DomBuilder.build(new StringReader(
                    ELUtils.getStringValue(elContext, sourceConfiguration.toString())));

            if (targetConfiguration == null)
            {
                return sourceConfigurationDOM;
            }

            return Xpp3Dom.mergeXpp3Dom(sourceConfigurationDOM, (Xpp3Dom) targetConfiguration);
        }
        catch (final Exception e)
        {
            throw new RuntimeException(e.getMessage(), e);
        }
    }
}
